import json
from datetime import datetime
from enum import Enum
from typing import Any, Dict, List, Self

from pydantic import AnyHttpUrl, BaseModel

from counsel.models.syft import SyftModel


class Severity(str, Enum):
    UNKNOWN = "unknown"
    NEGLIGIBLE = "negligible"
    LOW = "low"
    MEDIUM = "medium"
    HIGH = "high"
    CRITICAL = "critical"

    @classmethod
    def threshold(cls, severity: Self) -> List[Self]:
        severities = list(iter(cls))
        return severities[severities.index(severity) : :]


class ContainerStatus(str, Enum):
    RESTARTING = "restarting"
    RUNNING = "running"
    PAUSED = "paused"
    EXITED = "exited"


class Image(BaseModel):
    id: str
    labels: Dict[str, Any]
    tags: List[str]
    short_id: str


class Container(BaseModel):
    id: str
    name: str
    image: Image
    labels: Dict[str, Any]
    status: ContainerStatus
    short_id: str


class BOM(SyftModel):
    pass


class Artifact(BaseModel):
    name: str
    version: str
    type: str


class Vulnerability(BaseModel):
    id: str
    source: AnyHttpUrl
    artifact: Artifact
    namespace: str
    severity: Severity
    description: str | None


class VulnerabilitiesSummary(BaseModel):
    image_id: str
    tags: List[str]
    issuing_date: datetime = datetime.now()
    vulnerabilities: List[Vulnerability]

    @classmethod
    def parse_grype_dict(
        cls, grype_dict: dict[str, Any], issuing_date: datetime | None = None
    ) -> "VulnerabilitiesSummary":
        if issuing_date is None:
            issuing_date = datetime.now()

        summary = {
            "image_id": grype_dict["source"]["target"]["imageID"],
            "tags": grype_dict["source"]["target"]["tags"],
            "vulnerabilities": [],
            "issuing": issuing_date,
        }

        # Extract CVEs
        for m in grype_dict["matches"]:
            artifact = {
                "name": m["artifact"]["name"],
                "version": m["artifact"]["version"],
                "type": m["artifact"]["version"],
            }

            summary["vulnerabilities"].append(
                {
                    "id": m["vulnerability"]["id"],
                    "source": m["vulnerability"]["dataSource"],
                    "artifact": artifact,
                    "namespace": m["vulnerability"]["namespace"],
                    "severity": m["vulnerability"]["severity"].lower(),
                    "description": m["vulnerability"].get("description"),
                }
            )

        return cls.parse_obj(summary)

    @classmethod
    def parse_grype_json(cls, grype_str: str) -> "VulnerabilitiesSummary":
        return cls.parse_grype_dict(json.loads(grype_str))



# generated by datamodel-codegen:
#   filename:  syft-schema-7.0.1.json
#   timestamp: 2023-03-15T11:18:16+00:00

from __future__ import annotations

from datetime import datetime
from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel, Field, constr


class Descriptor(BaseModel):
    name: str
    version: str
    configuration: Optional[Any] = None


class Relationship(BaseModel):
    parent: str
    child: str
    type: str
    metadata: Optional[Any] = None


class Schema(BaseModel):
    version: str
    url: str


class Source(BaseModel):
    id: str
    type: str
    target: Any


class CargoPackageMetadata(BaseModel):
    name: str
    version: str
    source: str
    checksum: str
    dependencies: List[str]


class CocoapodsMetadata(BaseModel):
    checksum: str


class ConanLockMetadata(BaseModel):
    ref: str
    package_id: Optional[str] = None
    prev: Optional[str] = None
    requires: Optional[str] = None
    build_requires: Optional[str] = None
    py_requires: Optional[str] = None
    options: Optional[Dict[constr(regex=r".*"), str]] = None
    path: Optional[str] = None
    context: Optional[str] = None


class ConanMetadata(BaseModel):
    ref: str


class Coordinates(BaseModel):
    path: str
    layerID: Optional[str] = None


class DartPubMetadata(BaseModel):
    name: str
    version: str
    hosted_url: Optional[str] = None
    vcs_url: Optional[str] = None


class Digest(BaseModel):
    algorithm: str
    value: str


class DotnetDepsMetadata(BaseModel):
    name: str
    version: str
    path: str
    sha512: str
    hashPath: str


class FileMetadataEntry(BaseModel):
    mode: int
    type: str
    linkDestination: Optional[str] = None
    userID: int
    groupID: int
    mimeType: str


class GemMetadata(BaseModel):
    name: str
    version: str
    files: Optional[List[str]] = None
    authors: Optional[List[str]] = None
    licenses: Optional[List[str]] = None
    homepage: Optional[str] = None


class GolangBinMetadata(BaseModel):
    goBuildSettings: Optional[Dict[constr(regex=r".*"), str]] = None
    goCompiledVersion: str
    architecture: str
    h1Digest: Optional[str] = None
    mainModule: Optional[str] = None


class GolangModMetadata(BaseModel):
    h1Digest: Optional[str] = None


class HackageMetadata(BaseModel):
    name: str
    version: str
    pkgHash: Optional[str] = None
    snapshotURL: Optional[str] = None


class IDLikes(BaseModel):
    __root__: List[str]


class KbPackageMetadata(BaseModel):
    product_id: str
    kb: str


class MixLockMetadata(BaseModel):
    name: str
    version: str
    pkgHash: str
    pkgHashExt: str


class NpmPackageJSONMetadata(BaseModel):
    name: str
    version: str
    files: Optional[List[str]] = None
    author: str
    licenses: List[str]
    homepage: str
    description: str
    url: str
    private: bool


class NpmPackageLockJSONMetadata(BaseModel):
    resolved: str
    integrity: str


class PythonPipfileLockMetadata(BaseModel):
    hashes: List[str]
    index: str


class RebarLockMetadata(BaseModel):
    name: str
    version: str
    pkgHash: str
    pkgHashExt: str


class SearchResult(BaseModel):
    classification: str
    lineNumber: int
    lineOffset: int
    seekPosition: int
    length: int
    value: Optional[str] = None


class AlpmFileRecord(BaseModel):
    path: Optional[str] = None
    type: Optional[str] = None
    uid: Optional[str] = None
    gid: Optional[str] = None
    time: Optional[datetime] = None
    size: Optional[str] = None
    link: Optional[str] = None
    digest: Optional[List[Digest]] = None


class ApkFileRecord(BaseModel):
    path: str
    ownerUid: Optional[str] = None
    ownerGid: Optional[str] = None
    permissions: Optional[str] = None
    digest: Optional[Digest] = None


class DpkgFileRecord(BaseModel):
    path: str
    digest: Optional[Digest] = None
    isConfigFile: bool


class JavaManifest(BaseModel):
    main: Optional[Dict[constr(regex=r".*"), str]] = None
    namedSections: Optional[
        Dict[constr(regex=r".*"), Dict[constr(regex=r".*"), str]]
    ] = None


class PhpComposerAuthors(BaseModel):
    name: str
    email: Optional[str] = None
    homepage: Optional[str] = None


class PhpComposerExternalReference(BaseModel):
    type: str
    url: str
    reference: str
    shasum: Optional[str] = None


class PomProperties(BaseModel):
    path: str
    name: str
    groupId: str
    artifactId: str
    version: str
    extraFields: Optional[Dict[constr(regex=r".*"), str]] = None


class PortageFileRecord(BaseModel):
    path: str
    digest: Optional[Digest] = None


class PythonDirectURLOriginInfo(BaseModel):
    url: str
    commitId: Optional[str] = None
    vcs: Optional[str] = None


class RpmdbFileRecord(BaseModel):
    path: str
    mode: int
    size: int
    digest: Digest
    userName: str
    groupName: str
    flags: str


class Location(BaseModel):
    path: str
    layerID: Optional[str] = None
    virtualPath: Optional[str] = None


class PomParent(BaseModel):
    groupId: str
    artifactId: str
    version: str


class PythonFileDigest(BaseModel):
    algorithm: str
    value: str


class File(BaseModel):
    id: str
    location: Coordinates
    metadata: Optional[FileMetadataEntry] = None
    contents: Optional[str] = None
    digests: Optional[List[Digest]] = None


class LinuxRelease(BaseModel):
    prettyName: Optional[str] = None
    name: Optional[str] = None
    id: Optional[str] = None
    idLike: Optional[IDLikes] = None
    version: Optional[str] = None
    versionID: Optional[str] = None
    versionCodename: Optional[str] = None
    buildID: Optional[str] = None
    imageID: Optional[str] = None
    imageVersion: Optional[str] = None
    variant: Optional[str] = None
    variantID: Optional[str] = None
    homeURL: Optional[str] = None
    supportURL: Optional[str] = None
    bugReportURL: Optional[str] = None
    privacyPolicyURL: Optional[str] = None
    cpeName: Optional[str] = None
    supportEnd: Optional[str] = None


class Secrets(BaseModel):
    location: Coordinates
    secrets: List[SearchResult]


class AlpmMetadata(BaseModel):
    basepackage: str
    package: str
    version: str
    description: str
    architecture: str
    size: int
    packager: str
    license: str
    url: str
    validation: str
    reason: int
    files: List[AlpmFileRecord]
    backup: List[AlpmFileRecord]


class ApkMetadata(BaseModel):
    package: str
    originPackage: str
    maintainer: str
    version: str
    license: str
    architecture: str
    url: str
    description: str
    size: int
    installedSize: int
    pullDependencies: List[str]
    provides: List[str]
    pullChecksum: str
    gitCommitOfApkPort: str
    files: List[ApkFileRecord]


class DpkgMetadata(BaseModel):
    package: str
    source: str
    version: str
    sourceVersion: str
    architecture: str
    maintainer: str
    installedSize: int
    files: List[DpkgFileRecord]


class PhpComposerJSONMetadata(BaseModel):
    name: str
    version: str
    source: PhpComposerExternalReference
    dist: PhpComposerExternalReference
    require: Optional[Dict[constr(regex=r".*"), str]] = None
    provide: Optional[Dict[constr(regex=r".*"), str]] = None
    require_dev: Optional[Dict[constr(regex=r".*"), str]] = Field(
        None, alias="require-dev"
    )
    suggest: Optional[Dict[constr(regex=r".*"), str]] = None
    type: Optional[str] = None
    notification_url: Optional[str] = Field(None, alias="notification-url")
    bin: Optional[List[str]] = None
    license: Optional[List[str]] = None
    authors: Optional[List[PhpComposerAuthors]] = None
    description: Optional[str] = None
    homepage: Optional[str] = None
    keywords: Optional[List[str]] = None
    time: Optional[str] = None


class PortageMetadata(BaseModel):
    installedSize: int
    files: List[PortageFileRecord]


class RpmMetadata(BaseModel):
    name: str
    version: str
    epoch: Optional[int]
    architecture: str
    release: str
    sourceRpm: str
    size: int
    license: str
    vendor: str
    modularityLabel: str
    files: List[RpmdbFileRecord]


class ClassifierMatch(BaseModel):
    classifier: str
    location: Location


class PomProject(BaseModel):
    path: str
    parent: Optional[PomParent] = None
    groupId: str
    artifactId: str
    version: str
    name: str
    description: Optional[str] = None
    url: Optional[str] = None


class PythonFileRecord(BaseModel):
    path: str
    digest: Optional[PythonFileDigest] = None
    size: Optional[str] = None


class BinaryMetadata(BaseModel):
    matches: List[ClassifierMatch]


class JavaMetadata(BaseModel):
    virtualPath: str
    manifest: Optional[JavaManifest] = None
    pomProperties: Optional[PomProperties] = None
    pomProject: Optional[PomProject] = None
    digest: Optional[List[Digest]] = None


class PythonPackageMetadata(BaseModel):
    name: str
    version: str
    license: str
    author: str
    authorEmail: str
    platform: str
    files: Optional[List[PythonFileRecord]] = None
    sitePackagesRootPath: str
    topLevelPackages: Optional[List[str]] = None
    directUrlOrigin: Optional[PythonDirectURLOriginInfo] = None


class Package(BaseModel):
    id: str
    name: str
    version: str
    type: str
    foundBy: str
    locations: List[Coordinates]
    licenses: List[str]
    language: str
    cpes: List[str]
    purl: str
    metadataType: Optional[str] = None
    metadata: Optional[
        Union[
            AlpmMetadata,
            ApkMetadata,
            BinaryMetadata,
            CargoPackageMetadata,
            CocoapodsMetadata,
            ConanLockMetadata,
            ConanMetadata,
            DartPubMetadata,
            DotnetDepsMetadata,
            DpkgMetadata,
            GemMetadata,
            GolangBinMetadata,
            GolangModMetadata,
            HackageMetadata,
            JavaMetadata,
            KbPackageMetadata,
            MixLockMetadata,
            NpmPackageJSONMetadata,
            NpmPackageLockJSONMetadata,
            PhpComposerJSONMetadata,
            PortageMetadata,
            PythonPackageMetadata,
            PythonPipfileLockMetadata,
            RebarLockMetadata,
            RpmMetadata,
        ]
    ] = None


class Document(BaseModel):
    artifacts: List[Package]
    artifactRelationships: List[Relationship]
    files: Optional[List[File]] = None
    secrets: Optional[List[Secrets]] = None
    source: Source
    distro: LinuxRelease
    descriptor: Descriptor
    schema_: Schema = Field(..., alias="schema")


class Model(BaseModel):
    __root__: Document


class SyftModel(Model):
    pass



import importlib
import logging
from datetime import datetime, timedelta
from typing import List

import apprise
from jinja2 import Template
from pydantic import AnyHttpUrl, BaseModel, FilePath

from counsel.models.counsel import Severity, VulnerabilitiesSummary

logger = logging.getLogger(__name__)


# TODO Consider passing issuing policy (so actions can alter the policy, e.g., cleanup).
class Action(BaseModel):
    def __call__(self, vulnerability_summary: VulnerabilitiesSummary) -> None:
        self.run(vulnerability_summary)

    def run(self, vulnerability_summary: VulnerabilitiesSummary) -> None:
        logger.info(f"Running action for {vulnerability_summary.image_id}")
        logger.info(
            f'  - Vulnerabilities found for "{vulnerability_summary.tags[0]}": {len(vulnerability_summary.vulnerabilities)}'  # noqa: E501
        )
        logger.info("  - High/Critical Vulnerabilities:")
        for v in vulnerability_summary.vulnerabilities:
            if v.severity.value in ["high", "critical"]:
                logger.info(
                    f"      - { v.id} - {v.artifact.name}/{v.artifact.version} ({v.severity.value}) - {v.source}"  # noqa: E501
                )


class SlackNotification(Action):
    slack_webhook_url: AnyHttpUrl
    template_path: FilePath = importlib.resources.files(  # type: ignore
        "counsel.assets.templates"
    ).joinpath("slack.md.j2")

    def run(self, vulnerability_summary: VulnerabilitiesSummary) -> None:
        logger.info(f"Running slack action for: {vulnerability_summary.image_id}")
        logger.info(f"Reading Jinja2 template from file: {self.template_path}")

        with open(self.template_path) as f:
            template = Template(f.read())

        msg = self.render_template(vulnerability_summary, template)
        apobj = apprise.Apprise(f"{self.slack_webhook_url}?footer=no")

        logger.info(f"Notifying: {self.slack_webhook_url}")

        apobj.notify(body=msg)

    @classmethod
    def render_template(
        cls, vulnerability_summary: VulnerabilitiesSummary, template: Template
    ) -> str:
        logger.info("Rendering Jinja2 template")
        return template.render(
            vulnerabilities=vulnerability_summary.vulnerabilities,
            tags=vulnerability_summary.tags,
            image_id=vulnerability_summary.image_id,
        )


# TODO Implement ways to keep the old/pending issuing_date
# (so running policies are kept between image updates).
# TODO Remove from history if images has diapeared
# (e.g. if image as not be  proceesed for a while.).
class Policy(BaseModel):
    """A policy is applied based as soon as the constrains are met."""

    name: str
    severities: List[Severity] = Severity.threshold(Severity.HIGH)
    action: Action = Action()
    delay: timedelta | None = None  # timedelta(days=14)
    description: str | None = None

    execution_history: List[str] = []

    def check_policy(self, vulnerabilities_summary: VulnerabilitiesSummary) -> bool:
        logger.info(f"Checking policy for: {vulnerabilities_summary.tags[0]}")
        # sourcery skip: use-any, use-next
        # Check agains history:
        history_pass = False
        if vulnerabilities_summary.image_id in self.execution_history:
            logger.info("  - History check: FAIL")
            return False
        else:
            logger.info("  - History check: PASS")
            history_pass = True

        # Validate delta:
        delay_pass = False
        if self.delay and datetime.now() < (
            vulnerabilities_summary.issuing_date + self.delay
        ):
            logger.info("  - Delay check: FAIL")
            return False
        else:
            logger.info("  - Delay check: PASS")
            delay_pass = True

        # Validate severity:πP
        severity_pass = False
        for v in vulnerabilities_summary.vulnerabilities:
            if v.severity in self.severities:
                logger.info("  - Severity check: PASS")
                severity_pass = True
                break
        if not severity_pass:
            logger.info("  - Severity check: FAIL")

        return history_pass and delay_pass and severity_pass

    def check_policies(
        self, vulnerabilities_summaries: List[VulnerabilitiesSummary]
    ) -> List[bool]:
        return [self.check_policy(v) for v in vulnerabilities_summaries]

    def apply_policy(
        self, vulnerabilities_summary: VulnerabilitiesSummary, force: bool = False
    ) -> bool:
        logger.info(
            f"Attempting to apply policy for: {vulnerabilities_summary.tags[0]}"
        )
        if self.check_policy(vulnerabilities_summary) or force:
            self.action(vulnerabilities_summary)

            if vulnerabilities_summary.image_id not in self.execution_history:
                self.execution_history.append(vulnerabilities_summary.image_id)

            return True

        return False

    def apply_policies(
        self,
        vulnerabilities_summaries: List[VulnerabilitiesSummary],
        force: bool = False,
    ) -> List[bool]:
        return [self.apply_policy(v, force=force) for v in vulnerabilities_summaries]



import logging
import os
import sched
import tempfile
import time
from datetime import datetime
from subprocess import run
from typing import List, Tuple

import cachetools
import docker
from cachetools import Cache, TTLCache, cachedmethod
from croniter import croniter
from pydantic import BaseModel, FilePath, PrivateAttr

from counsel.models.counsel import BOM, Container, Image, VulnerabilitiesSummary
from counsel.policy import Policy

logger = logging.getLogger(__name__)


class Counsel(BaseModel):
    syft_path: FilePath = "syft"
    grype_path: FilePath = "grype"

    scan_all_schedule: str | None = None
    policies: List[Policy] = []
    # cache: Optional[FilePath] = None

    _docker_client: docker.models.configs.ConfigCollection = PrivateAttr(
        default_factory=docker.from_env
    )
    _methods_cache: Cache = PrivateAttr()  # type: ignore

    def __init__(self, **data) -> None:  # type: ignore
        super().__init__(**data)
        self._methods_cache = TTLCache(maxsize=32, ttl=60)

    def scan(
        self,
        image_id: str,
        apply_policies: bool = False,
        force_policies: bool = False,
    ) -> VulnerabilitiesSummary:
        v = self.vulnerabilities(image_id)
        if apply_policies:
            self.apply_policies(v.image_id, force_policies=force_policies)

        return v

    def scan_all(
        self, apply_policies: bool = False, force_policies: bool = False
    ) -> List[VulnerabilitiesSummary]:
        return [
            self.scan(
                i.id,
                apply_policies=apply_policies,
                force_policies=force_policies,
            )
            for i in self.images()
        ]

    def check_policies(
        self,
        image_id: str,
    ) -> Tuple[VulnerabilitiesSummary, List[Tuple[Policy, bool]]]:
        v = self.scan(image_id, apply_policies=False, force_policies=False)
        p = [p.check_policy(v) for p in self.policies]

        return v, list(zip(self.policies, p))

    def check_policies_all(
        self,
    ) -> List[Tuple[VulnerabilitiesSummary, List[Tuple[Policy, bool]]]]:
        return [
            self.check_policies(
                i,
            )
            for i in self.images()
        ]

    def apply_policies(
        self, image_id: str, force_policies: bool = False
    ) -> Tuple[VulnerabilitiesSummary, List[Tuple[Policy, bool]]]:
        v = self.scan(image_id=image_id, apply_policies=False, force_policies=False)
        p = [p.apply_policy(v, force=force_policies) for p in self.policies]

        return v, list(zip(self.policies, p))

    def apply_policies_all(
        self, force_policies: bool = False
    ) -> List[Tuple[VulnerabilitiesSummary, List[Tuple[Policy, bool]]]]:
        return [
            self.apply_policies(i, force_policies=force_policies) for i in self.images()
        ]

    def _containers(self) -> Tuple[List[Container], List[Image]]:
        client = self._docker_client

        # Get running containers (by default only show running containers):
        images = {}
        countainers_list = []

        for container in client.containers.list():
            image = container.image

            i = Image(
                id=image.id,
                labels=image.labels,
                tags=image.tags,
                short_id=image.short_id,
            )

            c = Container(
                id=container.id,
                name=container.name,
                image=i,
                labels=container.labels,
                status=container.status,
                short_id=container.short_id,
            )

            images[i.id] = i
            countainers_list.append(c)

        return countainers_list, list(images.values())

    def containers(self) -> List[Container]:
        return self._containers()[0]

    def images(self) -> List[Image]:
        return self._containers()[1]

    @cachedmethod(
        lambda self: self._methods_cache, key=cachetools.keys.methodkey  # type: ignore
    )  # noaq E501
    def bom(self, image_id: str) -> BOM:
        logger.info(f"Getting BOOM for: {image_id}")

        data = run(
            [self.syft_path, "--quiet", "-o", "json", image_id],
            capture_output=True,
        )
        # data = json.loads(data.stdout)

        return BOM.parse_raw(data.stdout)

    @cachedmethod(
        lambda self: self._methods_cache, key=cachetools.keys.methodkey  # type: ignore
    )  # noaq E501
    def vulnerabilities(self, image_id: str) -> VulnerabilitiesSummary:
        # Analyse vulnerabilities using Grype:
        # Create temporary file for analysis:
        logger.info(f"Getting vulnerabilities for: {image_id}")
        v = VulnerabilitiesSummary(image_id=image_id, tags=[], vulnerabilities=[])
        bom = self.bom(image_id)

        fd, path = tempfile.mkstemp(suffix=".json")
        try:
            with os.fdopen(fd, "w") as tmp:
                tmp.write(bom.json(by_alias=True))

            data = run(
                [self.grype_path, "--quiet", "-o", "json", path],
                capture_output=True,
            )

            v = VulnerabilitiesSummary.parse_grype_json(data.stdout)  # type: ignore

        finally:
            os.remove(path)

        return v

    def run_schedule(
        self, initial_scan: bool = False, apply_policies: bool = True
    ) -> None:
        cron_itr = croniter(self.scan_all_schedule)  # type: ignore
        scheduler = sched.scheduler(timefunc=time.time)

        logger.info(f"Cron schedule is: '{self.scan_all_schedule}'")

        if initial_scan:
            logger.info("Running initial scan before scheduling (initial_scan=True).")
            self.scan_all(apply_policies=apply_policies)
        try:
            while True:
                t = cron_itr.get_next(datetime)
                logger.info(f"The next scan is scheduled at: {t}.")
                scheduler.enterabs(
                    t.timestamp(),
                    1,
                    self.scan_all,
                    kwargs={"apply_policies": apply_policies},
                )
                scheduler.run()
        except KeyboardInterrupt:
            logger.info("Stopping scheduler")
        finally:
            for e in scheduler.queue:
                scheduler.cancel(e)


if __name__ == "__main__":
    c = Counsel()
    c.scan_all()



import logging
from datetime import timedelta

import typer
import urllib3
from rich.logging import RichHandler

import counsel
from counsel.counsel import Counsel
from counsel.models.counsel import Severity
from counsel.policy import Policy, SlackNotification

logging.basicConfig(
    level="NOTSET",
    format="%(message)s",
    datefmt="[%X]",
    handlers=[RichHandler(rich_tracebacks=True, tracebacks_suppress=[typer, urllib3])],
)
logging.getLogger("docker").setLevel(logging.WARNING)
logging.getLogger("urllib3").setLevel(logging.WARNING)
logging.getLogger("apprise").setLevel(logging.WARNING)
logging.getLogger("asyncio").setLevel(logging.WARNING)

logger = logging.getLogger(__name__)
logger.setLevel(level="INFO")

app = typer.Typer()


@app.command()
def callback(
    scan_schedule: str = typer.Option(
        "", envvar="COUNSEL_SCAN_SHEDULE", help="Schedule when to run Counsel."
    ),
    initial_scan: bool = typer.Option(
        False,
        "--initial-scan",
        envvar="COUNSEL_INITIAL_SCAN",
        help="Run an initial scan before starting the scheduler.",
    ),
    alert_threshold: Severity = typer.Option(
        default="high",
        case_sensitive=False,
        envvar="COUNSEL_THRESHOLD",
        help="Minimum vulnerability threshold for raising an alert.",
    ),
    remind_delay: int = typer.Option(
        default=10,
        envvar="COUNSEL_REMIND_DELAY",
        help="When a notification/action reminder should be sent (in days).",
    ),
    kill_delay: int = typer.Option(
        default=14,
        envvar="COUNSEL_KILL_DELAY",
        help="When the final notification/action should be sent (in days).",
    ),
    slack_webhook_url: str = typer.Option(
        default="",
        envvar="COUNSEL_SLACK_URL",
        help="Slack webhook URL for alert notifications.",
    ),
    slack_msg_template: str = typer.Option(
        default="",
        envvar="COUNSEL_SLACK_MSG_TEMPLATE_PATH",
        help="Slack Jinja2 template for alert notifications.",
    ),
) -> None:
    """🕵️‍♀️ Counsel - A tool for monitoring the vulnerabilities of docker containers."""  # noqa E501

    logger.info(f"Running Counsel version: {counsel.__version__}")
    logger.info("Parameters:")
    if scan_schedule:
        logger.info(f"  - COUNSEL_SCAN_SHEDULE: {scan_schedule}")
    logger.info(f"  - COUNSEL_THRESHOLD: {alert_threshold.value}")
    logger.info(f"  - COUNSEL_REMIND_DELAY: {remind_delay}")

    if slack_webhook_url:
        logger.info(f"  - COUNSEL_SLACK_URL: {slack_webhook_url}")

    if slack_msg_template:
        logger.info(f"  - COUNSEL_SLACK_MSG_TEMPLATE_PATH: {slack_webhook_url}")

    # logger.info(logging.Logger.manager.loggerDict)

    cs = Counsel()
    if scan_schedule != "":
        cs.scan_all_schedule = scan_schedule

    # TODO: Allow parameter for policies, probably using a configuration file
    severity_threshold = Severity.threshold(alert_threshold)
    cs.policies.append(Policy(name="warn", severities=severity_threshold))
    cs.policies.append(
        Policy(
            name="remind",
            severities=severity_threshold,
            delay=timedelta(days=remind_delay),
        )
    )
    cs.policies.append(
        Policy(
            name="kill",
            severities=severity_threshold,
            delay=timedelta(days=kill_delay),
        )
    )

    if slack_webhook_url != "":
        slack = SlackNotification(slack_webhook_url=slack_webhook_url)
        if slack_msg_template != "":
            slack.template_path = slack_msg_template

        cs.policies.append(
            Policy(name="warn/slack", severities=severity_threshold, action=slack)
        )
        cs.policies.append(
            Policy(
                name="remind/slack",
                severities=severity_threshold,
                delay=timedelta(days=remind_delay),
                action=slack,
            )
        )
        cs.policies.append(
            Policy(
                name="kill/slack",
                severities=severity_threshold,
                delay=timedelta(days=kill_delay),
                action=slack,
            )
        )

    logger.info("Policies")
    for pol in cs.policies:
        logger.info(
            f"  - {pol.name} at {str(pol.delay or 'Now')}: {str(pol.description or '')} ({pol.action.__class__.__name__})"  # noqa: E501
        )

    if scan_schedule != "":
        cs.run_schedule(initial_scan=initial_scan)
    else:
        cs.scan_all(apply_policies=True)


if __name__ == "__main__":
    app()
